# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['tap_clickup', 'tap_clickup.tests']

package_data = \
{'': ['*'], 'tap_clickup': ['schemas/*']}

install_requires = \
['requests>=2.25.1,<3.0.0', 'singer-sdk==0.3.13']

entry_points = \
{'console_scripts': ['tap-clickup = tap_clickup.tap:TapClickUp.cli']}

setup_kwargs = {
    'name': 'tap-clickup',
    'version': '0.0.14',
    'description': 'tap-clickup is a Singer tap for ClickUp by AutoIDM',
    'long_description': "# `tap-clickup` ![Build and Tests](https://github.com/AutoIDM/tap-clickup/actions/workflows/ci.yml/badge.svg?branch=main) [![PyPI download month](https://img.shields.io/pypi/dm/tap-clickup.svg)](https://pypi.python.org/pypi/tap-clickup/) \n`tap-clickup` is a Singer tap for ClickUp.\n\n## Capabilities\n\n* `sync`\n* `catalog`\n* `state`\n* `discover`\n\n## Settings\n\n| Setting   | Required | Default | Description |\n|:----------|:--------:|:-------:|:------------|\n| api_token | True     | None    | Example: 'pk_12345'            |\n\nA full list of supported settings and capabilities is available by running: `tap-clickup --about`\n\n### Getting an API Token\n\n1. Login at https://app.clickup.com/\n1. Click your icon at the bottom left of the screen\n1. Click My Settings\n1. Click Apps (Bottom leftish of screen under the My Apps sub section)\n1. At the top of the screen is an API Token. This can be used in the api_token location. \n    * This is a personal token, it's fine to use a personal token as this tap is only for the business that's using the data. \n\n## Clickup Replication\nStart Date is used for the initial updated at value for the updated at field with tasks. \n\nLet's say that you only want tasks that have been updated in the last year. To accomplish this you would pass in a start date of the first of this year!\n\n## Clickup Table Schemas\nNote that the most up to date information is located in tap_clickup/streams.py. We will try to keep these docs updated\n\n### Teams\n- Table name: team\n- Description: Teams Data, each user can be in multiple teams\n- Primary key column(s):  id\n- Replicated fully or incrementally: No\n- Bookmark column(s): N/A\n- Link to API endpoint documentation: [Teams](https://jsapi.apiary.io/apis/clickup20/reference/0/teams/get-teams.html)\n\n### Spaces\n- Table name: space\n- Description: Each team has multiple spaces\n- Primary key column(s):  id\n- Replicated fully or incrementally: No\n- Bookmark column(s): N/A\n- Link to API endpoint documentation: [Spaces](https://jsapi.apiary.io/apis/clickup20/reference/0/spaces.html)\n\n### Folders\n- Table name: folder\n- Description: Each space can have multiple folders\n- Primary key column(s):  id\n- Replicated fully or incrementally: No\n- Bookmark column(s): N/A\n- Link to API endpoint documentation: [Folders](https://jsapi.apiary.io/apis/clickup20/reference/0/folders.html)\n\n### Folder Lists\n- Table name: folder_list\n- Description: Each Folder can have multiple lists\n- Primary key column(s):  id\n- Replicated fully or incrementally: No\n- Bookmark column(s): N/A\n- Link to API endpoint documentation:[Folder Lists](https://jsapi.apiary.io/apis/clickup20/reference/0/lists/get-lists.html)\n\n\n### Folderless Lists\n- Table name: folderless_list\n- Description: Some lists do not exist in a folder\n- Primary key column(s):  id\n- Replicated fully or incrementally: No\n- Bookmark column(s): N/A\n- Link to API endpoint documentation: [Folderless Lists](https://jsapi.apiary.io/apis/clickup20/reference/0/lists/get-folderless-lists.html)\n\n### Task Templates\n- Table name: task_template\n- Description: Tasks can be templated for any reason you can imagine!\n- Primary key column(s):  id\n- Replicated fully or incrementally: No\n- Bookmark column(s): N/A\n- Link to API endpoint documentation: [Task Templates](https://jsapi.apiary.io/apis/clickup20/reference/0/task-templates/get-task-templates.html)\n\n### Goals\n- Table name: goal \n- Description: Teams can set goals for themselves\n- Primary key column(s):  id\n- Replicated fully or incrementally: No\n- Bookmark column(s): N/A\n- Link to API endpoint documentation: [Goals](https://jsapi.apiary.io/apis/clickup20/reference/0/goals/get-goals.html)\n\n### Tags\n- Table name: tag\n- Description: Each space can have multiple tags\n- Primary key column(s):  id\n- Replicated fully or incrementally: No\n- Bookmark column(s): N/A\n- Link to API endpoint documentation: [Tags](https://jsapi.apiary.io/apis/clickup20/reference/0/tags/get-space-tags.html)\n\n### Shared Hierarchy\n- Table name: shared_hierarchy\n- Description: Returns all resources you have access to where you don't have access to its parent. For example, if you have a access to a shared task, but don't have access to its parent list, it will come back in this request.\n- Primary key column(s): (No primary key column)\n- Replicated fully or incrementally: No\n- Bookmark column(s): N/A\n- Link to API endpoint documentation: [Shared Hierarchy](https://jsapi.apiary.io/apis/clickup20/reference/0/shared-hierarchy/shared-hierarchy.html)\n\n### Custom Fields from Folderless Lists\n- Table name: folderless_customfield\n- Description: Each Folderless lists can have custom fields associated with them\n- Primary key column(s):  id\n- Replicated fully or incrementally: No\n- Bookmark column(s): N/A\n- Link to API endpoint documentation: [Custom Field](https://jsapi.apiary.io/apis/clickup20/reference/0/custom-fields/get-accessible-custom-fields.html)\n\n### Custom Field from Folder Lists\n- Table name: folder_customfield\n- Description: Each Foldere list can have custom fields associated with them\n- Primary key column(s):  id\n- Replicated fully or incrementally: No\n- Bookmark column(s): N/A\n- Link to API endpoint documentation: [Custom Field](https://jsapi.apiary.io/apis/clickup20/reference/0/custom-fields/get-accessible-custom-fields.html)\n\n### Tasks\n- Table name: tasks\n- Description: Some tasks do not sit under folders. This comes from the folderless_list endpoint\n- Primary key column(s):  id\n- Replicated fully or incrementally: No\n- Bookmark column(s): date_updated. Note that the api endpoint date_updated_gt is great than or equal to, not just greater than. \n- Link to API endpoint documentation: [Get Tasks](https://jsapi.apiary.io/apis/clickup20/reference/0/tasks/get-filtered-team-tasks.html)\n\n## Other Info\n\n* Dates are returned in UNIX time \n* API Limiting uses [X-RateLimit headers](https://tools.ietf.org/id/draft-polli-ratelimit-headers-00.html)\n\n## Installation\n\n```bash\npipx install tap-clickup\n```\n\n## Usage\n\nYou can easily run `tap-clickup` by itself or in a pipeline using [Meltano](https://meltano.com/).\n\n### Executing the Tap Directly\n\n```bash\ntap-clickup --version\ntap-clickup --help\ntap-clickup --config CONFIG --discover > ./catalog.json\n```\n\n## Developer Resources\n\n### Initialize your Development Environment\n\n```bash\npipx install poetry\npoetry install\n```\n\n### Create and Run Tests\n\nCreate tests within the `tap_clickup/tests` subfolder and\n  then run:\n\n```bash\npoetry run pytest\n```\n\nYou can also test the `tap-clickup` CLI interface directly using `poetry run`:\n\n```bash\npoetry run tap-clickup --help\n```\n\n### Testing with [Meltano](https://www.meltano.com)\n\n_**Note:** This tap will work in any Singer environment and does not require Meltano.\n\nInstall Meltano (if you haven't already) and any needed plugins:\n\n```bash\n# Install meltano\npipx install meltano\n# Initialize meltano within this directory\ncd tap-clickup\nmeltano install\n```\n\nNow you can test and orchestrate using Meltano:\n\n```bash\n# Test invocation:\nmeltano invoke tap-clickup --version\n# OR run a test `elt` pipeline:\nmeltano elt tap-clickup target-jsonl\n```\n\n### SDK\n\nBuilt with the [Meltano SDK](https://sdk.meltano.com) for Singer Taps and Targets.\n\n",
    'author': 'AutoIDM',
    'author_email': None,
    'maintainer': 'AutoIDM',
    'maintainer_email': None,
    'url': 'https://autoidm.com',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6.2,<3.10',
}


setup(**setup_kwargs)
